all_chain_release_details = {
    "nndc": {
        "b7.1": {
            "chain": {
                "url": "https://raw.githubusercontent.com/openmc-data-storage/openmc_data/849229a66060240a09b45236f3d8d43f0434235b/openmc_data/depletion/chain-nndc-b7.1.xml",
            }
        },
    }
}
