all_h5_release_details = {
    "nndc": {
        "b7.1": {
            "neutron-photon": {
                "base_url": "https://anl.box.com/shared/static/",
                "compressed_files": ["9igk353zpy8fn9ttvtrqgzvw1vtejoz6.xz"],
                "compressed_file_size": 1.6,
                "uncompressed_file_size": 6.2,
            }
        },
    }
}
